"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const nunjucks = require("nunjucks");
const fs_1 = require("fs");
function engine(overland, ctx) {
    // return a render fn.
    return function (filename, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof filename !== 'string') {
                data = filename;
                filename = null;
            }
            // i.e., we've not been provided a specific file.
            if (filename === null) {
                filename = `${ctx.controller.name}/${ctx.controller.action}.html`;
            }
            // fetch our application
            const application = overland.apps.get(ctx.controller.app);
            // configure load paths
            const paths = [
                overland.settings.views,
                `${overland.settings.views}/${ctx.controller.app}`,
                application.views
            ];
            // stat potential template files
            const candidates = yield Promise.all(paths.map((viewDir) => {
                return new Promise((resolve, reject) => {
                    const test = `${viewDir}/${filename}`;
                    fs_1.stat(test, (err, res) => {
                        if (err)
                            resolve(false);
                        resolve(true);
                    });
                });
            }));
            const base = paths[candidates.findIndex(p => p === true)];
            if (base === undefined) {
                const e = new Error(`No template found with name '${filename}' in app or site directory!`);
                e.name = 'No Such Template';
                throw e;
            }
            const env = nunjucks.configure(paths, { watch: true });
            const renderEnv = application.nunjucksEnv ? Object.assign(application.nunjucksEnv, { loaders: env.loaders, opts: env.opts }) : env;
            renderEnv.addGlobal('css', ctx.state.css);
            renderEnv.addGlobal('js', ctx.state.js);
            renderEnv.addGlobal('asset', ctx.state.asset_path);
            const renderCtx = overland.createRenderContext(ctx, data);
            return renderEnv.render(base + '/' + filename, renderCtx);
        });
    };
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = engine;
